"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const TcpNet_1 = require("../TcpNet");
const events_1 = require("events");
const debug_1 = require("debug");
const Time_1 = require("../Time");
const proto_1 = require("./proto");
const zlib_1 = __importDefault(require("zlib"));
const Util_1 = require("../Util");
let Debug = debug_1.debug('server-libs');
class GroupMgr {
    constructor(host, port) {
        this.host = '';
        this.port = 0;
        this.selfInfo = { roleId: '', score: 0, info: {} };
        this.groupId = '';
        this.groupInfo = { selfRole: proto_1.RoleType.normal, roles: {} };
        this.event = new events_1.EventEmitter();
        //-------------------------------------//
        this.hellback = {};
        this.host = host;
        this.port = port;
    }
    static create(host, port, roleId, groupId, score, info) {
        let group = new GroupMgr(host, port);
        group.joinGroup(roleId, groupId, score, info);
        return group;
    }
    init() {
        return new Promise((resolve, reject) => {
            this.socket = TcpNet_1.TcpSocket.create(this.host, this.port);
            TcpNet_1.TcpSocket.connect(this.socket).then(() => {
                this.onConnection();
                resolve();
            }).catch(reject);
        });
    }
    onConnection() {
        this.socket.removeAllListeners('connection');
        this.socket.on('close', this.onClose.bind(this));
        this.socket.on('data', this.onData.bind(this));
        this.socket.on("error", () => { });
        this.event.emit("connection");
    }
    get isMaster() {
        return this.groupInfo.selfRole == proto_1.RoleType.master;
    }
    // 查询某个节点的角色
    getRole(roleId) {
        for (let key in this.groupInfo.roles) {
            if (!this.groupInfo.roles[key])
                continue;
            if (this.groupInfo.roles[key].indexOf(roleId) >= 0) {
                return parseInt(key);
            }
        }
        return proto_1.RoleType.normal;
    }
    get masterRoleId() {
        return this.groupInfo.roles[proto_1.RoleType.master][0];
    }
    get roleId() {
        return this.selfInfo.roleId;
    }
    onsayhellotiemout() {
        try {
            // 通知所有节点失败的消息
            let pool = this.hellback.b || [];
            for (let i = 0; i < pool.length; i++) {
                pool[i][1]();
            }
        }
        catch (e) {
        }
        this.hellback.b = [];
    }
    // 通知client 应答状态 主要是查询master状态
    sayhello(data) {
        if (!this._pushData(proto_1.eMasterC2SType.sayhello, JSON.stringify(data)))
            return Promise.reject();
        return new Promise((resolve, reject) => {
            if (!this.hellback.b) {
                this.hellback.b = [[resolve, reject]];
            }
            else {
                this.hellback.b.push([resolve, reject]);
            }
            if (!this.hellback.t) {
                this.hellback.t = setTimeout(this.onsayhellotiemout.bind(this), 5000);
            }
        });
    }
    //------------------------------------------//
    onData(cmd, data) {
        let info;
        try {
            info = JSON.parse(data.toString());
        }
        catch (e) {
            console.error(e, 'cmd', cmd);
            return;
        }
        switch (cmd) {
            // 注册信息
            case proto_1.eMasterS2CType.join:
                this.onJoin(info.roleId, info.role, info.groupId);
                break;
            case proto_1.eMasterS2CType.talk:
                this.onTalk(info.srcId, info);
                break;
            case proto_1.eMasterS2CType.groupInit:
                this.onInit(info.groupId, info.roles);
                break;
            case proto_1.eMasterS2CType.roleResult:
                this.onRoleResult(info.groupId, info.roles);
                break;
            case proto_1.eMasterS2CType.leave:
                this.onLeave(info.roleId, info.groupId, info.role);
                break;
            case proto_1.eMasterS2CType.groupInitFail:
                // 重复了，就要提示用户
                this.event.emit("error", info.addr);
                break;
            case proto_1.eMasterS2CType.sayhello:
                this._pushData(proto_1.eMasterC2SType.sayhelloback, JSON.stringify({
                    t: Time_1.LocalDate.now(),
                    role: this.getRole(this.roleId)
                }));
                break;
            case proto_1.eMasterS2CType.sayhelloback: {
                try {
                    // 移除超时
                    if (this.hellback.t) {
                        clearTimeout(this.hellback.t);
                        delete this.hellback.t;
                    }
                    // 通知所有节点成功的消息
                    let pool = this.hellback.b || [];
                    for (let i = 0; i < pool.length; i++) {
                        try {
                            pool[i][0](info);
                        }
                        catch (e) {
                            if (typeof pool[i][1] == 'function')
                                pool[i][1](e);
                        }
                    }
                }
                catch (e) {
                }
                this.hellback.b = [];
                break;
            }
            default:
                break;
        }
    }
    // 断开的时候尝试一下解除
    onClose(socketaddr, isException) {
        let _this = this;
        // 把自己的group信息干掉
        // 把自己的数据设置为普通，防止master状态下执行
        this.groupInfo.selfRole = proto_1.RoleType.normal;
        // 清理掉所有人的角色信息
        for (let key in this.groupInfo.roles) {
            let pool = this.groupInfo.roles[key];
            if (pool)
                pool.splice(0, pool.length);
        }
        if (isException) {
            // 断开的时候需要 重新发起连接
            function connect() {
                TcpNet_1.TcpSocket.connect(_this.socket).then(_this.onConnection.bind(_this)).catch(() => {
                    Debug("net close:", socketaddr, "ready to auto reconnect");
                    process.nextTick(connect);
                });
            }
            connect();
        }
        // 上报自己的工作状态为断开
        this.event.emit("close");
    }
    close() {
        this.event.removeAllListeners();
        this.socket.removeAllListeners();
        this.socket.close();
    }
    // 加入分组
    joinGroup(roleId, groupId, score, info) {
        this.selfInfo.roleId = roleId;
        this.selfInfo.info = info;
        this.groupId = groupId;
        this.selfInfo.score = score || this.selfInfo.score;
        if (this.handle)
            this.handle.stop();
        this.handle = Util_1.Util.setInterval(this.updateSayhello.bind(this), 10 * 1000);
        return this._pushData(proto_1.eMasterC2SType.join, JSON.stringify({
            groupId: this.groupId,
            roleId: this.selfInfo.roleId,
            info: Object.assign({ score: this.selfInfo.score }, this.selfInfo.info)
        }));
    }
    _pushData(cmd, data) {
        return this.socket.sendMsg(cmd, data);
    }
    // 推送点对点的消息
    pushMsg(targetId, type, cmd, info, roleId, extId) {
        let s_info = JSON.stringify(info);
        let zib = false;
        if (s_info.length > 256) {
            s_info = zlib_1.default.deflateRawSync(s_info);
            zib = true;
        }
        let talkUnit = {
            type: type,
            cmd: cmd,
            info: s_info,
            zib: zib,
            roleId: roleId,
            extId: extId
        };
        return this._pushData(proto_1.eMasterC2SType.talk, JSON.stringify({
            groupId: this.groupId,
            roleId: this.selfInfo.roleId,
            targetId: targetId,
            info: talkUnit
        }));
    }
    // 群发消息
    pushBroadcast(type, cmd, info, roleId, extId) {
        let s_info = JSON.stringify(info);
        let zib = false;
        if (s_info.length > 256) {
            s_info = zlib_1.default.deflateRawSync(s_info);
            zib = true;
        }
        let talkUnit = {
            type: type,
            cmd: cmd,
            info: s_info,
            zib: zib,
            roleId: roleId,
            extId: extId
        };
        return this._pushData(proto_1.eMasterC2SType.brodcast, JSON.stringify({
            groupId: this.groupId,
            roleId: this.selfInfo.roleId,
            info: talkUnit
        }));
    }
    updateSayhello() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.groupId)
                return true;
            try {
                let result = yield this.sayhello({
                    groupId: this.groupId,
                    roleId: this.roleId,
                    t: Time_1.LocalDate.now()
                });
                this.parseRole(result.roles);
            }
            catch (e) {
                console.log(e);
            }
            return true;
        });
    }
    // 收到分组初始化信息 
    onInit(groupId, roles) {
        this.groupId = groupId;
        this.parseRole(roles);
        // 上报自己状态为连接
        this.event.emit('open');
        this.event.emit('ready');
    }
    parseRole(roles) {
        let preRole = this.groupInfo.selfRole;
        this.groupInfo.roles = roles;
        let find = false;
        for (let key in roles) {
            if (!roles[key])
                continue;
            if (roles[key].indexOf(this.selfInfo.roleId) >= 0) {
                this.groupInfo.selfRole = parseInt(key);
                find = true;
                break;
            }
        }
        if (!find) {
            this.groupInfo.selfRole = proto_1.RoleType.normal;
        }
        // 这里对于选举结果要处理一下，如果自己的角色定位改变了需要额外通知一下
        if (preRole != this.groupInfo.selfRole) {
            this.event.emit("roletype", this.groupInfo.selfRole == proto_1.RoleType.master, this.groupInfo.selfRole);
        }
    }
    // 有人加入
    onJoin(roleId, role, groupId) {
        if (this.groupId != groupId)
            return;
        if (!this.groupInfo.roles[role]) {
            this.groupInfo.roles[role] = [roleId];
        }
        else {
            this.groupInfo.roles[role].push(roleId);
        }
    }
    // 有人离开
    onLeave(roleId, groupId, role) {
        if (this.groupId != groupId)
            return;
        if (this.groupInfo.roles[role]) {
            let idx = this.groupInfo.roles[role].push(roleId);
            this.groupInfo.roles[role].splice(idx, 1);
        }
    }
    // 重新选举了
    onRoleResult(groupId, roles) {
        if (this.groupId != groupId) {
            console.log('onRoleResult groupid is diff ', 'this:' + this.groupId, 'server:' + groupId);
            return;
        }
        this.groupId = groupId;
        this.parseRole(roles);
    }
    onTalk(roleId, info) {
        //  type: string, cmd: number, info: any, extId: string
        if (info.zib) {
            // 压缩过的需要解压缩
            info.info = zlib_1.default.inflateRawSync(Buffer.from(info.info));
        }
        info.info = JSON.parse(info.info.toString());
        this.event.emit('data', roleId, info.type, info.cmd, info.info, info.extId);
    }
    on(event, listener) { this.event.on(event, listener); return this; }
    once(event, listener) { this.event.once(event, listener); return this; }
    off(event, listener) { this.event.off(event, listener); return this; }
}
exports.GroupMgr = GroupMgr;
